package com.tzld.piaoquan.tencentad.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.tencentad.common.base.CommonResponse;
import com.tzld.piaoquan.tencentad.common.base.ExceptionCodeEnum;
import com.tzld.piaoquan.tencentad.common.constants.TimeConstant;
import com.tzld.piaoquan.tencentad.common.enums.AdMuseImageStatusEnum;
import com.tzld.piaoquan.tencentad.common.enums.AdMuseTaskStatusEnum;
import com.tzld.piaoquan.tencentad.common.enums.AdVideoStatusEnum;
import com.tzld.piaoquan.tencentad.common.enums.SimilarityCodeEnum;
import com.tzld.piaoquan.tencentad.component.HttpPoolClient;
import com.tzld.piaoquan.tencentad.dao.mapper.*;
import com.tzld.piaoquan.tencentad.model.bo.ImageParam;
import com.tzld.piaoquan.tencentad.model.po.*;
import com.tzld.piaoquan.tencentad.model.vo.AdMuseImageParam;
import com.tzld.piaoquan.tencentad.model.vo.AdMuseImageVo;
import com.tzld.piaoquan.tencentad.model.vo.AdVideoImageParam;
import com.tzld.piaoquan.tencentad.service.AdAccountService;
import com.tzld.piaoquan.tencentad.service.AdMuseTaskService;
import com.tzld.piaoquan.tencentad.utils.DateUtils;
import com.tzld.piaoquan.tencentad.utils.ImageUtils;
import com.tzld.piaoquan.tencentad.utils.OSSUploader;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.List;

@Slf4j
@Service
public class AdMuseTaskServiceImpl implements AdMuseTaskService {

    @Value("${image_path}")
    private String imagePath;

    @Autowired
    private HttpPoolClient httpPoolClient;

    @Autowired
    private AdMuseTaskMapper adMuseTaskMapper;

    @Autowired
    private AdVideoImageMapper adVideoImageMapper;

    @Autowired
    private AdAccountService accountService;

    @Autowired
    private AdMuseImageMapper adMuseImageMapper;

    public void addMuseTask(String accountId, String accessToken, AdVideoImageParam adVideoImageParam, AdVideoImage adVideoImage) {
        try {
            String url = String.format("https://api.e.qq.com/v3.0/muse_ai_task/add?access_token=%s&timestamp=%s&nonce=%s",
                    accessToken, System.currentTimeMillis() / 1000, UUID.randomUUID().toString().replaceAll("-", ""));
            JSONObject param = new JSONObject();
            param.put("account_id", accountId);
            if (StringUtils.isNotEmpty(adVideoImageParam.getText2imgDesc())) {
                param.put("task_type", "MUSE_AI_TASK_TYPE_TEXT2IMG");
                JSONObject text2img = new JSONObject();
                if (StringUtils.isEmpty(adVideoImageParam.getDimensionSize())) {
                    text2img.put("dimension_size", adVideoImageParam.getDimensionSize());
                } else {
                    text2img.put("dimension_size", "SIZE_1280_720");
                }
                text2img.put("prompt", adVideoImageParam.getText2imgDesc());
                param.put("text2img", text2img);
            } else {
                param.put("task_type", "MUSE_AI_TASK_TYPE_IMG2IMG");
                JSONObject img2img = new JSONObject();
                //默认取4级相似
                img2img.put("image_id", adVideoImage.getTencentImageId());
                img2img.put("similarity", adVideoImageParam.getSimilarity() == null ?
                        SimilarityCodeEnum.FOURTH.getCode() : adVideoImageParam.getSimilarity());
                img2img.put("img2img_style", "MUSE_AI_IMG2IMG_STYLE_PEOPLE");
                if (StringUtils.isNotEmpty(adVideoImageParam.getDimensionSize())) {
                    img2img.put("dimension_size", adVideoImageParam.getDimensionSize());
                } else {
                    img2img.put("extend_size", true);
                }
                param.put("img2img", img2img);
            }
            String res = httpPoolClient.post(url, param.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(res);
            if (jsonObject != null && jsonObject.getInteger("code") == 0) {
                JSONObject data = jsonObject.getJSONObject("data");
                AdMuseTask adMuseTask = new AdMuseTask();
                adMuseTask.setTaskId(data.getLong("task_id"));
                adMuseTask.setTaskStatus(data.getString("task_status"));
                adMuseTask.setTaskType(data.getString("task_type"));
                adMuseTask.setAccountId(accountId);
                adMuseTask.setAdVideoId(adVideoImage.getAdVideoId());
                adMuseTask.setAdVideoImageId(adVideoImage.getId());
                adMuseTask.setSubtitle(adVideoImageParam.getSubtitle());
                adMuseTaskMapper.insertSelective(adMuseTask);
            }
        } catch (Exception e) {
            log.error("createMuseTask error ", e);
        }
    }

    public List<AdMuseTask> getAdMuseTask(Integer status) {
        AdMuseTaskExample example = new AdMuseTaskExample();
        example.createCriteria().andStatusEqualTo(status).andIsDeleteEqualTo(0);
        return adMuseTaskMapper.selectByExample(example);
    }

    public List<AdMuseTask> getAdMuseTask(Long adVideoId, Integer status, List<Long> adMuseTaskIds) {
        AdMuseTaskExample example = new AdMuseTaskExample();
        example.createCriteria().andAdVideoIdEqualTo(adVideoId).andStatusEqualTo(status).andIdIn(adMuseTaskIds);
        return adMuseTaskMapper.selectByExample(example);
    }

    public boolean getMuseTaskRes(AdMuseTask adMuseTask, String accessToken) {
        try {
            long now = System.currentTimeMillis();
            long create = adMuseTask.getCreateTime().getTime();
            if (now < create + 3 * TimeConstant.MILLISECOND_MINUTE) {
                return false;
            }
            long timestamp = System.currentTimeMillis() / 1000;
            String url = String.format("https://api.e.qq.com/v3.0/muse_ai_task/get?access_token=%s&timestamp=%s&nonce=%s" +
                            "&account_id=%s&task_id=%s",
                    accessToken, timestamp, timestamp, adMuseTask.getAccountId(), adMuseTask.getTaskId());
            String res = httpPoolClient.get(url);
            JSONObject jsonObject = JSONObject.parseObject(res);
            if (jsonObject != null && jsonObject.getInteger("code") == 0) {
                JSONObject data = jsonObject.getJSONObject("data");
                String taskStatus = data.getString("task_status");
                if ("MUSE_AI_TASK_SUCCESS".equals(taskStatus)) {
                    adMuseTask.setTaskStatus(taskStatus);
                    JSONArray results = data.getJSONArray("results");
                    for (int i = 0; i < results.size(); i++) {
                        JSONObject result = results.getJSONObject(i);
                        AdMuseImage adMuseImage = new AdMuseImage();
                        adMuseImage.setAdVideoImageId(adMuseTask.getAdVideoImageId());
                        adMuseImage.setAdVideoId(adMuseTask.getAdVideoId());
                        adMuseImage.setAdMuseTaskId(adMuseTask.getId());
                        adMuseImage.setMuseMaterialId(result.getLong("muse_material_id"));
                        adMuseImage.setMuseUrl(result.getString("url"));
                        adMuseImage.setStitchUrl(getStitchUrl(result.getString("url"), adMuseTask.getSubtitle()));
                        adMuseImageMapper.insertSelective(adMuseImage);
                    }
                    adMuseTask.setStatus(AdMuseTaskStatusEnum.CREATE_SUCCESS.getStatus());
                    adMuseTaskMapper.updateByPrimaryKeySelective(adMuseTask);
                    return true;
                }
                if ("MUSE_AI_TASK_FAIL".equals(taskStatus)) {
                    adMuseTask.setTaskStatus(taskStatus);
                    adMuseTask.setStatus(AdMuseTaskStatusEnum.CREATE_FAIL.getStatus());
                    adMuseTaskMapper.updateByPrimaryKeySelective(adMuseTask);
                    return true;
                }
            }
        } catch (Exception e) {
            log.error("createMuseTask error ", e);
        }
        return false;
    }

    public String getStitchUrl(String url, String subtitle) {
        if (StringUtils.isEmpty(url)) {
            return null;
        }
        String targetImgPath = imagePath + "/" + UUID.randomUUID() + ".jpg";
        ImageParam imageParam = new ImageParam();
        imageParam.setFont(new Font("微软雅黑", Font.PLAIN, 48));
        imageParam.setColor(new Color(102, 102, 102));
        imageParam.setText(subtitle);
        return ImageUtils.writeImageFromUrl(url, targetImgPath, imageParam);
    }

    public long selectCount(Long adVideoId, List<Integer> status) {
        AdMuseTaskExample example = new AdMuseTaskExample();
        AdMuseTaskExample.Criteria criteria = example.createCriteria().andAdVideoIdEqualTo(adVideoId);
        if (!CollectionUtils.isEmpty(status)) {
            criteria.andStatusIn(status);
        }
        return adMuseTaskMapper.countByExample(example);
    }

    @Override
    public CommonResponse<List<AdMuseImageVo>> getAdMuseImageVoList(Long adVideoId) {
        AdMuseImageExample example = new AdMuseImageExample();
        example.createCriteria().andAdVideoIdEqualTo(adVideoId);
        List<AdMuseImage> adMuseImages = adMuseImageMapper.selectByExample(example);
        List<AdMuseImageVo> res = new ArrayList<>();
        if (CollectionUtils.isEmpty(adMuseImages)) {
            return CommonResponse.success(res);
        }
        for (AdMuseImage adMuseImage : adMuseImages) {
            AdMuseImageVo adMuseImageVo = new AdMuseImageVo();
            BeanUtils.copyProperties(adMuseImage, adMuseImageVo);
            AdMuseImageStatusEnum adMuseImageStatusEnum = AdMuseImageStatusEnum.getByStatus(adMuseImage.getStatus());
            if (adMuseImageStatusEnum != null) {
                adMuseImageVo.setStatusName(adMuseImageStatusEnum.getDesc());
            }
            if (StringUtils.isNotEmpty(adMuseImage.getFinalUrl())) {
                adMuseImageVo.setStitchUrl(adMuseImage.getFinalUrl());
            }
            adMuseImageVo.setCreateTime(DateUtils.getDateString(adMuseImage.getCreateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            adMuseImageVo.setUpdateTime(DateUtils.getDateString(adMuseImage.getUpdateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            res.add(adMuseImageVo);
        }
        return CommonResponse.success(res);
    }


    public CommonResponse<Void> updateMuseImage(AdMuseImageParam adMuseImageParam) {
        if (adMuseImageParam == null || adMuseImageParam.getId() == null || adMuseImageParam.getStatus() == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误");
        }
        List<Integer> allowStatus = Arrays.asList(AdMuseImageStatusEnum.VERIFY_SUCCESS.getStatus(), AdMuseImageStatusEnum.VERIFY_FAIL.getStatus());
        if (!allowStatus.contains(adMuseImageParam.getStatus())) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，状态无效");
        }
        AdMuseImage adMuseImage = adMuseImageMapper.selectByPrimaryKey(adMuseImageParam.getId());
        if (adMuseImage == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，数据不存在");
        }
        adMuseImage.setStatus(adMuseImageParam.getStatus());
        adMuseImageMapper.updateByPrimaryKeySelective(adMuseImage);
        if (AdMuseImageStatusEnum.VERIFY_SUCCESS.getStatus().equals(adMuseImageParam.getStatus())) {
            AdVideoImage adVideoImage = adVideoImageMapper.selectByPrimaryKey(adMuseImage.getAdVideoImageId());
            String accountId = adVideoImage.getAccountId();
            String accessToken = accountService.getAccessToken(accountId);
            String url = StringUtils.isEmpty(adMuseImage.getFinalUrl()) ? adMuseImage.getStitchUrl() : adMuseImage.getFinalUrl();
            Long mediaId = uploadImage(accountId, accessToken, url, adMuseImage.getMuseMaterialId());
            if (mediaId != null) {
                adMuseImage.setMediaId(mediaId);
                adMuseImage.setStatus(AdMuseImageStatusEnum.UPLOAD_SUCCESS.getStatus());
                adMuseImageMapper.updateByPrimaryKeySelective(adMuseImage);
            } else {
                adMuseImage.setStatus(AdMuseImageStatusEnum.UPLOAD_FAIL.getStatus());
                adMuseImageMapper.updateByPrimaryKeySelective(adMuseImage);
            }
        }
        return CommonResponse.success();
    }

    @Override
    public CommonResponse<Void> uploadMuseImage(MultipartFile file, Long id) {
        if (id == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，id不能为空");
        }
        // 创建上传目录
        String fileName = file.getOriginalFilename();
        if (fileName == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，文件名无效");
        }
        File uploadDir = new File(imagePath);
        if (!uploadDir.exists()) {
            uploadDir.mkdirs(); // 创建目录
        }
        String path = imagePath + "/" + fileName;
        // 保存文件
        File destinationFile = new File(uploadDir, fileName);
        try {
            file.transferTo(destinationFile);
        } catch (IOException e) {
            return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "系统异常，上传失败");
        }
        String url = OSSUploader.uploadToOSS(path);
        if (StringUtils.isNotEmpty(url)) {
            AdMuseImage updateAdMuseImage = new AdMuseImage();
            updateAdMuseImage.setId(id);
            updateAdMuseImage.setFinalUrl(url);
            adMuseImageMapper.updateByPrimaryKeySelective(updateAdMuseImage);
            return CommonResponse.success();
        }
        return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "系统异常，上传失败");
    }

    @Override
    public List<AdMuseImage> getAdMuseImageList(List<Long> ids) {
        AdMuseImageExample example = new AdMuseImageExample();
        example.createCriteria().andIdIn(ids);
        return adMuseImageMapper.selectByExample(example);
    }

    public Long uploadImage(String accountId, String accessToken, String imageUrl, Long museMaterialId) {
        try {
            long timestamp = System.currentTimeMillis() / 1000;
            String url = String.format("https://api.e.qq.com/v3.0/muse_ai_ugc/add?access_token=%s&timestamp=%s&nonce=%s",
                    accessToken, timestamp, timestamp);
            JSONObject param = new JSONObject();
            param.put("account_id", accountId);
            param.put("muse_material_id", museMaterialId);
            param.put("media_base64_str", ImageUtils.imageUrlToBase64(imageUrl));
            String res = httpPoolClient.post(url, param.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(res);
            if (jsonObject != null && jsonObject.getInteger("code") == 0) {
                JSONObject data = jsonObject.getJSONObject("data");
                return data.getLong("media_id");
            }
        } catch (Exception e) {
            log.error("uploadImage error", e);
        }
        return null;
    }


}
