package com.tzld.piaoquan.tencentad.service.impl;

import com.tzld.piaoquan.tencentad.common.base.CommonResponse;
import com.tzld.piaoquan.tencentad.common.base.ExceptionCodeEnum;
import com.tzld.piaoquan.tencentad.common.enums.AdMuseTaskStatusEnum;
import com.tzld.piaoquan.tencentad.common.enums.AdVideoImageStatusEnum;
import com.tzld.piaoquan.tencentad.common.enums.AdVideoStatusEnum;
import com.tzld.piaoquan.tencentad.dao.mapper.AdVideoMapper;
import com.tzld.piaoquan.tencentad.model.po.*;
import com.tzld.piaoquan.tencentad.model.vo.AdCreativeParam;
import com.tzld.piaoquan.tencentad.model.vo.AdVideoParam;
import com.tzld.piaoquan.tencentad.model.vo.AdVideoVo;
import com.tzld.piaoquan.tencentad.service.*;
import com.tzld.piaoquan.tencentad.utils.DateUtils;
import com.tzld.piaoquan.tencentad.utils.FfmpegUtil;
import com.tzld.piaoquan.tencentad.utils.OSSUploader;
import com.tzld.piaoquan.tencentad.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.io.IOException;
import java.nio.file.*;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class AdVideoServiceImpl implements AdVideoService {

    @Value("${image_path}")
    private String imagePath;

    @Autowired
    private AdVideoImageService adVideoImageService;

    @Autowired
    private AdMuseTaskService adMuseTaskService;

    @Autowired
    private AdVideoMapper adVideoMapper;

    @Autowired
    private AdAccountService accountService;

    @Autowired
    private AdGroupService adGroupService;

    @Autowired
    private AdCreativeService adCreativeService;

    public void addAdVideo(AdBaseVideo adBaseVideo) {
        AdVideo adVideo = new AdVideo();
        adVideo.setVideoId(adBaseVideo.getVideoId());
        adVideo.setVideoPath(adBaseVideo.getVideoPath());
        adVideo.setVideoCover(adBaseVideo.getVideoCover());
        adVideoMapper.insertSelective(adVideo);
        buildVideoImage(adVideo);
    }


    @Override
    public CommonResponse<Page<AdVideoVo>> getAdVideoList(Integer pageNum, Integer pageSize, List<Integer> status) {
        Page<AdVideoVo> page = new Page<>();
        page.setCurrentPage(pageNum);
        page.setPageSize(pageSize);
        AdVideoExample example = new AdVideoExample();
        if (!CollectionUtils.isEmpty(status)) {
            example.createCriteria().andStatusIn(status);
        }
        example.setOrderByClause("create_time desc");
        example.setPage(page);
        List<AdVideo> adVideos = adVideoMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(adVideos)) {
            return CommonResponse.success(page);
        }
        List<AdVideoVo> adVideoVos = new ArrayList<>();
        for (AdVideo adVideo : adVideos) {
            AdVideoVo adVideoVo = new AdVideoVo();
            BeanUtils.copyProperties(adVideo, adVideoVo);
            AdVideoStatusEnum adVideoStatusEnum = AdVideoStatusEnum.getByStatus(adVideo.getStatus());
            if (adVideoStatusEnum != null) {
                adVideoVo.setStatusName(adVideoStatusEnum.getDesc());
            }
            adVideoVo.setCreateTime(DateUtils.getDateString(adVideo.getCreateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            adVideoVo.setUpdateTime(DateUtils.getDateString(adVideo.getUpdateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            adVideoVos.add(adVideoVo);
        }
        page.setObjs(adVideoVos);
        return CommonResponse.success(page);
    }

    @Override
    public CommonResponse<Void> createAdCreative(AdCreativeParam adCreativeParam) {
        if (adCreativeParam == null || adCreativeParam.getAdVideoId() == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误");
        }
        AdVideo adVideo = adVideoMapper.selectByPrimaryKey(adCreativeParam.getAdVideoId());
        if (adVideo == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，adVideoId不存在");
        }
        List<AdMuseImage> adMuseImages = adMuseTaskService.getAdMuseImageList(adCreativeParam.getAdMuseImageIds());
        if (CollectionUtils.isEmpty(adMuseImages)) {
            return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "系统错误，当前没有可以创建创意的素材");
        }
        List<Long> mediaIds = adMuseImages.stream().map(AdMuseImage::getMediaId).collect(Collectors.toList());
        boolean res = adCreativeService.addAdCreative(adVideo, mediaIds, adCreativeParam.getAdGroupId(),
                adCreativeParam.getAccountId(), adCreativeParam.getAdGroupName(), adCreativeParam.getAdMuseImageIds(),
                adCreativeParam.getDescription());
        if (res) {
            return CommonResponse.success();
        } else {
            return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "系统异常，创建失败");
        }
    }

    public static void main(String[] args) {
        String url = "http://rescdn.yishihui.com/longvideo/multitranscode/video/vpc/20191121/10426977qXdqkgYwJ8eJp8fFEb20200804131055256947746-1LD.mp4";
        String imageFolder = "/Users/shimeng/Desktop/image_path" + "/" + UUID.randomUUID();
        File dir = new File(imageFolder);
        if (!dir.exists()) {
            boolean mkdir = dir.mkdir();
            if (!mkdir) {
                return;
            }
        }
        String imagePath = imageFolder + "/" + "%04d.jpg";
        FfmpegUtil.getTargetThumbnail(url, imagePath);
        File[] fileList = dir.listFiles();
        if (fileList == null) {
            return;
        }
    }


    //抽取图片
    public void buildVideoImage(AdVideo adVideo) {
        String imageFolder = imagePath + "/" + UUID.randomUUID();
        File dir = new File(imageFolder);
        if (!dir.exists()) {
            boolean mkdir = dir.mkdir();
            if (!mkdir) {
                return;
            }
        }
        String imagePath = imageFolder + "/" + "%04d.jpg";
        FfmpegUtil.getTargetThumbnail(adVideo.getVideoPath(), imagePath);
        File[] fileList = dir.listFiles();
        if (fileList == null) {
            return;
        }
        int sum = 0;
        for (File file : fileList) {
            String imageUrl = OSSUploader.uploadToOSS(file.getPath());
            if (StringUtils.isEmpty(imageUrl)) {
                continue;
            }
            boolean addTag = adVideoImageService.addAdVideoImage(adVideo.getId(), imageUrl);
            if (addTag) {
                sum++;
            }
        }
        if (sum > 0) {
            updateAdVideoStatus(adVideo.getId(), AdVideoStatusEnum.CROP_SUCCESS.getStatus());
        } else {
            updateAdVideoRetry(adVideo);
            if (adVideo.getRetryCount() >= 2) {
                updateAdVideoStatus(adVideo.getId(), AdVideoStatusEnum.CROP_FAIL.getStatus());
            }
        }
        Path pathToDelete = Paths.get(imageFolder);
        try {
            // 使用 Files.walkFileTree 删除文件夹及其内容
            Files.walkFileTree(pathToDelete, new SimpleFileVisitor<Path>() {
                @Override
                public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
                    Files.delete(file); // 删除文件
                    return FileVisitResult.CONTINUE;
                }

                @Override
                public FileVisitResult postVisitDirectory(Path dir, IOException exc) throws IOException {
                    Files.delete(dir); // 删除空目录
                    return FileVisitResult.CONTINUE;
                }
            });
        } catch (IOException e) {
            log.error("del imageFolder error {}", e.getMessage());
        }
    }

//    @Override
//    public CommonResponse<Void> updateAdVideo(AdVideoParam adVideoParam) {
//        if (adVideoParam == null || adVideoParam.getId() == null || adVideoParam.getStatus() == null) {
//            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误");
//        }
//        List<Integer> allowStatus =
//                Arrays.asList(AdVideoStatusEnum.VERIFY_SUCCESS.getStatus(), AdVideoStatusEnum.VERIFY_FAIL.getStatus());
//        if (!allowStatus.contains(adVideoParam.getStatus())) {
//            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误");
//        }
//        AdVideo adVideo = adVideoMapper.selectByPrimaryKey(adVideoParam.getId());
//        if (adVideo == null) {
//            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，该视频不存在");
//        }
//        if (!Objects.equals(adVideo.getStatus(), AdVideoStatusEnum.CROP_SUCCESS.getStatus())) {
//            return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "该状态不允许修改审核状态");
//        }
//        if (Objects.equals(adVideoParam.getStatus(), AdVideoStatusEnum.VERIFY_SUCCESS.getStatus())) {
//            long count = adVideoImageService.selectCount(adVideoParam.getId(), AdVideoImageStatusEnum.VERIFY_SUCCESS.getStatus());
//            if (count <= 0) {
//                return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "没有审核通过的图片，不允许审核通过");
//            }
//            AdVideo updateAdVideo = new AdVideo();
//            updateAdVideo.setId(adVideoParam.getId());
//            updateAdVideo.setStatus(adVideoParam.getStatus());
//            updateAdVideo.setAccountId(adVideoParam.getAccountId());
//            adVideoMapper.updateByPrimaryKeySelective(updateAdVideo);
//
//        } else {
//            updateAdVideoStatus(adVideoParam.getId(), adVideoParam.getStatus());
//        }
//        return CommonResponse.success();
//    }

    @Override
    public void refreshMuseTaskStatus() {
        List<AdMuseTask> adMuseTasks = adMuseTaskService.getAdMuseTask(AdMuseTaskStatusEnum.DEFAULT.getStatus());
        for (AdMuseTask adMuseTask : adMuseTasks) {
            String accountId = adMuseTask.getAccountId();
            String accessToken = accountService.getAccessToken(accountId);
            adMuseTaskService.getMuseTaskRes(adMuseTask, accessToken);
        }
    }


    private void updateAdVideoStatus(Long id, Integer status) {
        AdVideo updateAdVideo = new AdVideo();
        updateAdVideo.setId(id);
        updateAdVideo.setStatus(status);
        adVideoMapper.updateByPrimaryKeySelective(updateAdVideo);
    }

    private void updateAdVideoRetry(AdVideo adVideo) {
        AdVideo updateAdVideo = new AdVideo();
        updateAdVideo.setId(adVideo.getId());
        updateAdVideo.setRetryCount(adVideo.getRetryCount() + 1);
        adVideoMapper.updateByPrimaryKeySelective(updateAdVideo);
    }


}
